#!/usr/bin/perl -I/usr/sausalito/perl
# $Id: beancounter.pl, v2.0.0-2 Fri 11 Nov 2011 06:31:11 AM CET mstauber Exp $
# Copyright 2006-2011 Stauber Multimedia Design. All rights reserved.

# This handler updates the data for all VPS and stores it into CCE.
# However, it fetches only the variable parameters such as 'held', 'maxheld' and 'failcnt'

use CCE;
my $cce = new CCE;
my $conf = '/var/lib/cobalt';

$cce->connectfd();
#$cce->connectuds(); # only for debugging

# Location of the vzlist-dump:
$vzlistdump = '/tmp/vserver-beancount';

# Location of vserver config files (directory - with trailing slash):
$vzcfgdir = '/etc/sysconfig/vz-scripts/';


##
# Find out which VPS-ID we need to update. Read CCE's VserverMasterNode / gui_list_id
##

($sys_oidmasternode) = $cce->find('VserverMasterNode');
($ok, $VserverMasterNode) = $cce->get($sys_oidmasternode);
$gui_list_id = $VserverMasterNode->{gui_list_id};

# Find out when the Vserver object was last updated:
@this_oid = $cce->find('Vserver', {'vpsid' => $gui_list_id});
($ok, $vps_systime) = $cce->get($this_oid[0]);
$last_update_time = $vps_systime->{timestamp};
$nowtime = time();

# Do a full update of all VPS data:
$do_full_update = "1";

##
# Generate vzlist-dump:
##

system("/usr/sbin/vzlist -a -o vpsid,ip,hostname,status,numproc,kmemsize,kmemsize.m,kmemsize.b,kmemsize.l,kmemsize.f,lockedpages,lockedpages.m,lockedpages.b,lockedpages.l,lockedpages.f,privvmpages,privvmpages.m,privvmpages.b,privvmpages.l,privvmpages.f,shmpages,shmpages.m,shmpages.b,shmpages.l,shmpages.f,numproc,numproc.m,numproc.b,numproc.l,numproc.f,physpages,physpages.m,physpages.b,physpages.l,physpages.f,vmguarpages,vmguarpages.m,vmguarpages.b,vmguarpages.l,vmguarpages.f,oomguarpages,oomguarpages.m,oomguarpages.b,oomguarpages.l,oomguarpages.f,numtcpsock,numtcpsock.m,numtcpsock.b,numtcpsock.l,numtcpsock.f,numflock,numflock.m,numflock.b,numflock.l,numflock.f,numpty,numpty.m,numpty.b,numpty.l,numpty.f,numsiginfo,numsiginfo.m,numsiginfo.b,numsiginfo.l,numsiginfo.f,tcpsndbuf,tcpsndbuf.m,tcpsndbuf.b,tcpsndbuf.l,tcpsndbuf.f,tcprcvbuf,tcprcvbuf.m,tcprcvbuf.b,tcprcvbuf.l,tcprcvbuf.f,othersockbuf,othersockbuf.m,othersockbuf.b,othersockbuf.l,othersockbuf.f,dgramrcvbuf,dgramrcvbuf.m,dgramrcvbuf.b,dgramrcvbuf.l,dgramrcvbuf.f,numothersock,numothersock.m,numothersock.b,numothersock.l,numothersock.f,dcachesize,dcachesize.m,dcachesize.b,dcachesize.l,dcachesize.f,numfile,numfile.m,numfile.b,numfile.l,numfile.f,numiptent,numiptent.m,numiptent.b,numiptent.l,numiptent.f,diskspace,diskspace.s,diskspace.h,diskinodes,diskinodes.s,diskinodes.h,laverage,cpulimit,cpuunits,swappages,swappages.m,swappages.b,swappages.l,swappages.f,hostname | sed 's/^[ \t]*//;s/[ \t]*\$//' > $vzlistdump");

# Pull the vzlist-dump into a Matrix:
#
# I LOVE matrix stuff! If there is a sledload of data to parse, 
# then there is hardly a more efficient way to do so.
# A legend of what colums the matrix has is at the end of the code.

open (F, $vzlistdump) || die "Could not open $vzlistdump: $!";
$a = 0; 				# Matrix line number, whereas 0 is the first (useable) line of the vzlist dump
while ($line = <F>) {

    chomp($line);
    next if $line =~ /^\s*$/; 		# skip blank lines
    if ($line =~ /^([A-Za-z]\w*)/) {
        #$matrix_name = $1;
        $matrix_name = 'VPSID';
    } 
    else {
        my (@row) = split (/\s+/, $line);
        push (@{$matrix_name}, \@row); 
    }
    if ($VPSID[$a][0]) { 			# Plausibility check. We will not store anything into CCE if 'vpsid' is not set.
	$currentvpsid = $VPSID[$a][0];
	if ($gui_list_id == $currentvpsid) {    # Only do actual work if this is the VPS we need to get info on 

	    # Fix for ultra-long hostnames:
	    $VPSID[$a][2] = $VPSID[$a][119];

	    # Check if VPS private area exists. Because it may not exist anymore, as the VPS could have been migrated off or manually have been deleted:
	    $vps_home_area = "/vz/private/$VPSID[$a][0]";
	    if ( -d $vps_home_area ) {
    		$VPSID[$a][131] = "1";
	    }
	    else {
    		$VPSID[$a][131] = "0";
	    }

	    &readconfig;			# Read missing info from vz config files
	    &feedthemonster;			# Feed info into CCE
	    if ($new_vps_found == '1') {
		&feedthemonster;		# We found a new VPS. We need to feed the monster again!
	    }
	}
    }

    $a++;
}
close(F);
system("/bin/rm $vzlistdump");


##
# Get info about the current CPU utilization and the power of the node:
##

# Location of the vzcpu-dump:
$vzcpudump = '/tmp/vserver-cpuinfo';

# Generate vzcpu-dump:
system("/usr/sbin/vzcpucheck | sed 's/^[ \t]*//;s/[ \t]*\$//' > $vzcpudump");

if (-e $vzcpudump) {
    $b = 0;
    open(CPUDUMP,"$vzcpudump") || die("Could not find $vzcpudump. $!");
    while ($line = <CPUDUMP>) {
        chomp($line);
        next if $line =~ /^\s*$/; # skip blank lines
        if (($line =~ /^Current CPU utilization: (.*)/) && ($b == 0)) {
            $CURRENT_CPU = $1;
        }
        if (($line =~ /^Power of the node: (.*)/) && ($b == 1)) {
            $NODE_CPU = $1;
        }
        $b++;
    }
    close(CPUDUMP);
    system("/bin/rm $vzcpudump");
    &cce_update_cpu; # Write the info into CCE
}

# There are a few options in config files that vzlist doesn't provide.
# So we need to read them from the respective config files in
# /etc/sysconfig/vz-scripts/*.conf

sub readconfig {
    # Does the config file exist?
    $cfile = "$vzcfgdir$VPSID[$a][0].conf";
    if (-e $cfile) {
	$VPSID[$currentvpsid][130] = "1";
    	open(CFILE,"$cfile") || die("Could not find $cfile. $!");
    	while ($line = <CFILE>) {
    	    chomp($line);
    	    next if $line =~ /^\s*$/; # skip blank lines

	    $onboot = "";
    	    if ($line =~ /^ONBOOT=\"([A-Za-z]\w*)\"/) {
    		$onboot = $1;
		if (($onboot eq 'Yes') || ($onboot eq 'yes')) {
    		    $VPSID[$currentvpsid][114] = 'Yes';
		}
		else {
    		    $VPSID[$currentvpsid][114] = 'No';
		}
	    }
    	    if ($line =~ /^CPUUNITS=\"(.*)\"/) {
    		$VPSID[$currentvpsid][113] = $1;
	    }
	    if ($line =~ /^VE_ROOT=\"(.*)\"/) {
		$VPSID[$currentvpsid][115] = $1;
		if ($VPSID[$currentvpsid][115] eq "") {
			$VPSID[$currentvpsid][115] = '/vz/root/$VEID';
		}
	    }
    	    if ($line =~ /^VE_PRIVATE=\"(.*)\"/) {
        	$VPSID[$currentvpsid][116] = $1;
		if ($VPSID[$currentvpsid][116] eq "") {
			$VPSID[$currentvpsid][116] = '/vz/private/$VEID';
		}
    	    }
    	    if ($line =~ /^OSTEMPLATE=\"(.*)\"/) {
        	$VPSID[$currentvpsid][117] = $1;
	    }
    	    if ($line =~ /^ORIGIN_SAMPLE=\"(.*)\"/) {
        	$VPSID[$currentvpsid][118] = $1;
	    }
	    if ($line =~ /^NAMESERVER=\"(.*)\"/) {
                $nameservers = $1;
                $nameservers =~ s/ /&/g;
                $VPSID[$currentvpsid][124] = "&" . $nameservers . "&";
    	    }
	    if ($line =~ /^QUOTATIME=\"(.*)\"/) {
    	    	$VPSID[$currentvpsid][125] = $1;
    	    }
	    $quota = '';
	    if ($line =~ /^DISK_QUOTA=\"(.*)\"/) {
		$quota = $1;
		if (($quota eq 'yes') || ($quota eq 'Yes')) {
    	    	    $VPSID[$currentvpsid][126] = 'Yes';
		}
		else {
    	    	    $VPSID[$currentvpsid][126] = 'No';
		}
    	    }
	    if (($line =~ /^DISK_QUOTA=(.*)/) && ($quota eq '')){
		$quota = $1;
		if (($quota eq 'yes') || ($quota eq 'Yes')) {
    	    	    $VPSID[$currentvpsid][126] = 'Yes';
		}
		else {
    	    	    $VPSID[$currentvpsid][126] = 'No';
		}
    	    }

	    if ($line =~ /^QUOTAUGIDLIMIT=\"(.*)\"/) {
    	    	$VPSID[$currentvpsid][127] = $1;
    	    }
	    if ($line =~ /^AVNUMPROC=\"(.*):(.*)\"/) {
		$VPSID[$currentvpsid][128] = $1;
    	    	$VPSID[$currentvpsid][129] = $2;
	    }
	    elsif ($line =~ /^AVNUMPROC=\"(.*)\"/) {
		$VPSID[$currentvpsid][128] = $1;
    	    	$VPSID[$currentvpsid][129] = $1;
	    }
	    # Handle case where AVNUMPROC is empty:
	    if ($VPSID[$currentvpsid][128] eq "") {
		$VPSID[$currentvpsid][128] = "0";
	    }
	    if ($VPSID[$currentvpsid][129] eq "") {
		$VPSID[$currentvpsid][129] = "0";
	    }
    	    
	}
    }
    else {
	$VPSID[$currentvpsid][130] = "0";
    }
    close(CFILE);

    # NAMESERVER must be of format 'ipAddrList', but on new VPS's it may not be set - yet. If it is
    # NOT set, we set it to '&127.0.0.1&' here:
    if (!$VPSID[$currentvpsid][124]) {
        $VPSID[$currentvpsid][124] = "&127.0.0.1&";
    }
    # QUOTATIME may not be set. If it's not set, we set it to zero:
    if (!$VPSID[$currentvpsid][125]) {
        $VPSID[$currentvpsid][125] = "0";
    }
    # DISK_QUOTA may not be set. Valid entries are "yes" or "no". If it's neither, we default to "no":
    if (($VPSID[$currentvpsid][126] ne "Yes") && ($VPSID[$currentvpsid][126] ne "No")) {
        $VPSID[$currentvpsid][126] = "No";
    }
    # QUOTAUGIDLIMIT may not be set. This is for second level quota (user and group disk quota inside the VPS)
    if (!$VPSID[$currentvpsid][127]) {
        $VPSID[$currentvpsid][127] = "10000";
    }
}

# Subroutine that feeds the data into CCE:
sub feedthemonster {
	&validate;
	# This is where we actually populate CCE with the gathered info:
        @oids = $cce->find('Vserver', {'vpsid' => $currentvpsid});
        if ($#oids < 0) {
                #print "Object not yet in CCE. Creating new one.\n";
                ($ok) = $cce->create('Vserver', {
                'vpsid' => $VPSID[$a][0],
                'ip' => $VPSID[$a][1],
                'hostname' => $VPSID[$a][2],
                'timestamp' => time()
                });
		$new_vps_found = '1'; 			# We found a new VPS, one that we have never seen before and which
							# is not yet in CCE. We do not want to risk to populate CCE with ALL
							# the found new data in one go. So let us just set the most basic info
							# and the timestamp. 
							#
							# We also set $new_vps_found = '1' to run feedthemonster() again.
        }
        else {
	    #print "Object already present in CCE. Updating it.\n";
            # find the system object
	    $new_vps_found = '0';
	    &validate;
            ($sys_oid) = $cce->find('Vserver', {'vpsid' => $currentvpsid});
            ($ok, $sys) = $cce->get($sys_oid);
		
	    # For debugging and logging purpose we do not write all 126 entries into CCE in one go. Instead we break it down
	    # into multiple blocks so that we can troubleshoot /var/log/messages easily for data-blocks with values that CCE
	    # did not like. 

	    # Always set these:                             
    	    ($ok) = $cce->set($sys_oid, '',{        # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'vpsid' => $VPSID[$a][0],                                                                                    
                'ip' => $VPSID[$a][1],                                                                                       
                'hostname' => $VPSID[$a][2],                                                                                 
                'status' => $VPSID[$a][3],                                                                                   
                'is_dir_present' => $VPSID[$a][131],                                                                         
                'timestamp' => time()                                                                                        
                });                                                                                                          

	    if ($do_full_update == "1") {
		# Only set these if the stored info is ancient:
    		($ok) = $cce->set($sys_oid, '',{        # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
            	    'diskinodes_s' => $VPSID[$a][109],                                                                           
            	    'diskinodes_h' => $VPSID[$a][110],                                                                           
            	    'cpulimit' => $VPSID[$a][112],                                                                               
            	    'cpuunits' => $VPSID[$a][113],                                                                               
            	    'timestamp' => time()                                                                                        
            	    });
            }                                                                                                          

	    if ($do_full_update == "1") {
		# Do not set these every time:
    	        ($ok) = $cce->set($sys_oid, '',{        # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
        	    'kmemsize_m' => $VPSID[$a][6],                                                                               
    		    'lockedpages_m' => $VPSID[$a][11],                                                                           
        	    'privvmpages_m' => $VPSID[$a][16],                                                                           
            	    'shmpages_m' => $VPSID[$a][21],                                                                              
            	    'numproc_m' => $VPSID[$a][26],                                                                               
            	    'physpages_m' => $VPSID[$a][31],                                                                             
            	    'swappages_m' => $VPSID[$a][115],                                                                             
            	    'vmguarpages_m' => $VPSID[$a][36],                                                                           
            	    'oomguarpages_m' => $VPSID[$a][41],                                                                          
            	    'numtcpsock_m' => $VPSID[$a][46],                                                                            
            	    'numflock_m' => $VPSID[$a][51],                                                                              
            	    'numpty_m' => $VPSID[$a][56],                                                                                
            	    'numsiginfo_m' => $VPSID[$a][61],
            	    'timestamp' => time()
            	    });
            }
	    if ($do_full_update == "1") {
		# Do not set these every time:
    	        ($ok) = $cce->set($sys_oid, '',{        # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
            	    'tcpsndbuf_m' => $VPSID[$a][66],
            	    'tcprcvbuf_m' => $VPSID[$a][71],
            	    'othersockbuf_m' => $VPSID[$a][76],
            	    'dgramrcvbuf_m' => $VPSID[$a][81],
            	    'numothersock_m' => $VPSID[$a][86],
            	    'dcachesize_m' => $VPSID[$a][91],
            	    'numfile_m' => $VPSID[$a][96],
            	    'numiptent_m' => $VPSID[$a][101],
            	    'diskspace_s' => $VPSID[$a][106],
            	    'diskspace_h' => $VPSID[$a][107],
            	    'timestamp' => time()
            	    });
            }

            # The next values are dynamical and dependent on VPS load and usage.
    	    ($ok) = $cce->set($sys_oid, '',{        # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'kmemsize' => $VPSID[$a][5],
                'lockedpages' => $VPSID[$a][10],
                'privvmpages' => $VPSID[$a][15],
                'shmpages' => $VPSID[$a][20],
                'numproc' => $VPSID[$a][25],
                'physpages' => $VPSID[$a][30],
                'swappages' => $VPSID[$a][114],
                'vmguarpages' => $VPSID[$a][35],
                'oomguarpages' => $VPSID[$a][40],
                'numtcpsock' => $VPSID[$a][45],
                'numflock' => $VPSID[$a][50],
                'numpty' => $VPSID[$a][55],
                'numsiginfo' => $VPSID[$a][60],
                'timestamp' => time()
                });
            # The next values are dynamical and dependent on VPS load and usage.
    	    ($ok) = $cce->set($sys_oid, '',{        # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'tcpsndbuf' => $VPSID[$a][65],
                'tcprcvbuf' => $VPSID[$a][70],
                'othersockbuf' => $VPSID[$a][75],
                'dgramrcvbuf' => $VPSID[$a][80],
                'numothersock' => $VPSID[$a][85],
                'dcachesize' => $VPSID[$a][90],
                'numfile' => $VPSID[$a][95],
                'numiptent' => $VPSID[$a][100],
                'diskspace' => $VPSID[$a][105],
                'diskinodes' => $VPSID[$a][108],
                'laverage' => $VPSID[$a][111],
                'timestamp' => time()
                });

	    # Failcounts:
    	    ($ok) = $cce->set($sys_oid, '',{        # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'kmemsize_f' => $VPSID[$a][9],
                'lockedpages_f' => $VPSID[$a][14],
                'privvmpages_f' => $VPSID[$a][19],
                'shmpages_f' => $VPSID[$a][24],
                'numproc_f' => $VPSID[$a][29],
                'physpages_f' => $VPSID[$a][34],
    		'swappages_f' => $VPSID[$a][118],                                                                             
                'vmguarpages_f' => $VPSID[$a][39],
                'oomguarpages_f' => $VPSID[$a][44],
                'numtcpsock_f' => $VPSID[$a][49],
                'numflock_f' => $VPSID[$a][54],
                'timestamp' => time()
                });

	    # Failcounts:
    	    ($ok) = $cce->set($sys_oid, '',{        # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'numpty_f' => $VPSID[$a][59],
                'numsiginfo_f' => $VPSID[$a][64],
                'tcpsndbuf_f' => $VPSID[$a][69],
                'tcprcvbuf_f' => $VPSID[$a][74],
                'othersockbuf_f' => $VPSID[$a][79],
                'dgramrcvbuf_f' => $VPSID[$a][84],
                'numothersock_f' => $VPSID[$a][89],
                'dcachesize_f' => $VPSID[$a][94],
                'numfile_f' => $VPSID[$a][99],
                'numiptent_f' => $VPSID[$a][104],
                'timestamp' => time()
                });
                
        }
}

# Validate the settings before we pass them on to CCE:
sub validate {

    # ip:
    if (($VPSID[$a][1] == 0) || ($VPSID[$a][1] eq "-") || ($VPSID[$a][1] eq "")) {
	$VPSID[$a][1] = "127.0.0.1";
    }
    # hostname:
    if (!$VPSID[$a][2]) {
	$VPSID[$a][2] = "localhost.localdomain";
    }

    ## If a VPS is stopped, then a LOT of values gathered from a VPS will be blanks, represented by a '-'. Which is an invalid
    ## character as far as our CCE-Schema is concerned. So we need to replace that with a '0'. But we cannot do that globally,
    ## as a VPS domain or hostname may also have a '-' in it. So we do that for each value independently:

    if ($VPSID[$a][3] ne "running") {
        $VPSID[$a][5] = "0";
        $VPSID[$a][6] = "0";
        $VPSID[$a][9] = "0";
        $VPSID[$a][10] = "0";
        $VPSID[$a][11] = "0";
        $VPSID[$a][14] = "0";
        $VPSID[$a][15] = "0";
        $VPSID[$a][16] = "0";
        $VPSID[$a][19] = "0";
        $VPSID[$a][20] = "0";
        $VPSID[$a][21] = "0";
        $VPSID[$a][24] = "0";
        $VPSID[$a][25] = "0";
        $VPSID[$a][26] = "0";
        $VPSID[$a][29] = "0";
        $VPSID[$a][30] = "0";
        $VPSID[$a][31] = "0";
        $VPSID[$a][34] = "0";
        $VPSID[$a][35] = "0";
        $VPSID[$a][36] = "0";
        $VPSID[$a][39] = "0";
        $VPSID[$a][40] = "0";
        $VPSID[$a][41] = "0";
        $VPSID[$a][44] = "0";
        $VPSID[$a][45] = "0";
        $VPSID[$a][46] = "0";
        $VPSID[$a][49] = "0";
        $VPSID[$a][50] = "0";
        $VPSID[$a][51] = "0";
        $VPSID[$a][54] = "0";
        $VPSID[$a][55] = "0";
        $VPSID[$a][56] = "0";
        $VPSID[$a][59] = "0";
        $VPSID[$a][60] = "0";
        $VPSID[$a][61] = "0";
        $VPSID[$a][64] = "0";
        $VPSID[$a][65] = "0";
        $VPSID[$a][66] = "0";
        $VPSID[$a][69] = "0";
        $VPSID[$a][70] = "0";
        $VPSID[$a][71] = "0";
        $VPSID[$a][74] = "0";
        $VPSID[$a][75] = "0";
        $VPSID[$a][76] = "0";
        $VPSID[$a][79] = "0";
        $VPSID[$a][80] = "0";
        $VPSID[$a][81] = "0";
        $VPSID[$a][84] = "0";
        $VPSID[$a][85] = "0";
        $VPSID[$a][86] = "0";
        $VPSID[$a][89] = "0";
        $VPSID[$a][90] = "0";
        $VPSID[$a][91] = "0";
        $VPSID[$a][94] = "0";
        $VPSID[$a][95] = "0";
        $VPSID[$a][96] = "0";
        $VPSID[$a][99] = "0";
        $VPSID[$a][100] = "0";
        $VPSID[$a][101] = "0";
        $VPSID[$a][104] = "0";
        $VPSID[$a][105] = "0";
        $VPSID[$a][108] = "0";
        $VPSID[$a][111] = "0";
	$VPSID[$a][114] = "0";
        $VPSID[$a][115] = "0";
        $VPSID[$a][118] = "0";
    }
        #$VPSID[$a][128] = "0";
        #$VPSID[$a][129] = "0";

}

# Update or populate CCE with the CPU_POWER and CURRENT_CPU (usage) of the master node:
sub cce_update_cpu {

	# This is where we actually populate CCE with the gathered info:
        @oids = $cce->find('VserverMasterNode');
        if ($#oids < 0) {
                #print "Object not yet in CCE. Creating new one.\n";
                ($ok) = $cce->create('VserverMasterNode', {
                'CURRENT_CPU' => $CURRENT_CPU,
                'NODE_CPU' => $NODE_CPU
                });
        }
        else {
                #print "Object already present in CCE. Updating it.\n";
                # find the system object
                ($sys_oid) = $cce->find('VserverMasterNode');
                ($ok, $sys) = $cce->get($sys_oid);
                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'CURRENT_CPU' => $CURRENT_CPU,
                'NODE_CPU' => $NODE_CPU
                });
	}
}


$cce->bye('SUCCESS');
exit(0);

# Legend of Matrix columns:

# 0 = vpsid
# 1 = ip
# 2 = hostname
# 3 = status
# 4 = numproc
# 5 = kmemsize
# 6 = kmemsize.m
# 7 = kmemsize.b
# 8 = kmemsize.l
# 9 = kmemsize.f
# 10 = lockedpages
# 11 = lockedpages.m
# 12 = lockedpages.b
# 13 = lockedpages.l
# 14 = lockedpages.f
# 15 = privvmpages
# 16 = privvmpages.m
# 17 = privvmpages.b
# 18 = privvmpages.l
# 19 = privvmpages.f
# 20 = shmpages
# 21 = shmpages.m
# 22 = shmpages.b
# 23 = shmpages.l
# 24 = shmpages.f
# 25 = numproc
# 26 = numproc.m
# 27 = numproc.b
# 28 = numproc.l
# 29 = numproc.f
# 30 = physpages
# 31 = physpages.m
# 32 = physpages.b
# 33 = physpages.l
# 34 = physpages.f
# 35 = vmguarpages
# 36 = vmguarpages.m
# 37 = vmguarpages.b
# 38 = vmguarpages.l
# 39 = vmguarpages.f
# 40 = oomguarpages
# 41 = oomguarpages.m
# 42 = oomguarpages.b
# 43 = oomguarpages.l
# 44 = oomguarpages.f
# 45 = numtcpsock
# 46 = numtcpsock.m
# 47 = numtcpsock.b
# 48 = numtcpsock.l
# 49 = numtcpsock.f
# 50 = numflock
# 51 = numflock.m
# 52 = numflock.b
# 53 = numflock.l
# 54 = numflock.f
# 55 = numpty
# 56 = numpty.m
# 57 = numpty.b
# 58 = numpty.l
# 59 = numpty.f
# 60 = numsiginfo
# 61 = numsiginfo.m
# 62 = numsiginfo.b
# 63 = numsiginfo.l
# 64 = numsiginfo.f
# 65 = tcpsndbuf
# 66 = tcpsndbuf.m
# 67 = tcpsndbuf.b
# 68 = tcpsndbuf.l
# 69 = tcpsndbuf.f
# 70 = tcprcvbuf
# 71 = tcprcvbuf.m
# 72 = tcprcvbuf.b
# 73 = tcprcvbuf.l
# 74 = tcprcvbuf.f
# 75 = othersockbuf
# 76 = othersockbuf.m
# 77 = othersockbuf.b
# 78 = othersockbuf.l
# 79 = othersockbuf.f
# 80 = dgramrcvbuf
# 81 = dgramrcvbuf.m
# 82 = dgramrcvbuf.b
# 83 = dgramrcvbuf.l
# 84 = dgramrcvbuf.f
# 85 = numothersock
# 86 = numothersock.m
# 87 = numothersock.b
# 88 = numothersock.l
# 89 = numothersock.f
# 90 = dcachesize
# 91 = dcachesize.m
# 92 = dcachesize.b
# 93 = dcachesize.l
# 94 = dcachesize.f
# 95 = numfile
# 96 = numfile.m
# 97 = numfile.b
# 98 = numfile.l
# 99 = numfile.f
# 100 = numiptent
# 101 = numiptent.m
# 102 = numiptent.b
# 103 = numiptent.l
# 104 = numiptent.f
# 105 = diskspace
# 106 = diskspace.s
# 107 = diskspace.h
# 108 = diskinodes
# 109 = diskinodes.s
# 110 = diskinodes.h
# 111 = laverage
# 112 = cpulimit
# 113 = cpuunits

### From config file:
# 119 = onboot
# 120 = ve_root
# 121 = ve_private
# 122 = ostemplate
# 123 = origin_sample
# 124 = nameserver
# 125 = quotatime
# 126 = disk_quota
# 127 = quotaugidlimit
# 128 = avnumproc_b
# 129 = avnumproc_l
# 130 = cfg file exists

### From file operations:
# 131 = is_dir_present (is VPS private area present?)

