#!/usr/bin/perl -I/usr/sausalito/perl
#
# $Id: create.pl,v 1.0.0.8 Wed 27 Apr 2011 01:10:43 PM CEST mstauber Exp $
# Copyright 2006-2011 Stauber Multimedia Design. All rights reserved.
#
# Handler which creates new VPS's.
#

use CCE;
use POSIX qw(isalpha);

$DEBUG = 0;

my $cce = new CCE('Domain' => 'base-vserver');

$cce->connectfd();

# Double check if the IP is not already assigned - kinda redundant, but never mind:

$vserver = $cce->event_object();
$vserver_new = $cce->event_new();
$vserver_old = $cce->event_old();

@oids= $cce->find("Vserver");
($ok, $system) = $cce->get($oids[0]);

$vpsid = $vserver_new->{vpsid};

# De-Scalarize $NAMESERVER:
$NAMESERVER = $vserver_new->{NAMESERVER};
@dns = split('&', $NAMESERVER);
shift(@dns);
$nameservers = join(" ", @dns);

if ($DEBUG == 1) {
    system("echo \"VPSID-debug: controlAction: $vserver_new->{controlAction}\" > /tmp/vps_create");
    system("echo \"VPSID-debug: vpsid: $vserver_new->{vpsid}\" >> /tmp/vps_create");
    system("echo \"VPSID-debug: vpsid: $vpsid\" >> /tmp/vps_create");
    system("echo \"VPSID-debug: OSTEMPLATE: $vserver_new->{OSTEMPLATE}\" >> /tmp/vps_create");
    system("echo \"VPSID-debug: ORIGIN_SAMPLE: $vserver_new->{ORIGIN_SAMPLE}\" >> /tmp/vps_create");
    system("echo \"VPSID-debug: IP: $vserver_new->{ip}\" >> /tmp/vps_create");
    system("echo \"VPSID-debug: Hostname: $vserver_new->{hostname}\" >> /tmp/vps_create");
    system("echo \"VPSID-debug: DNS: $nameservers\" >> /tmp/vps_create");
    system("echo \"VPSID-debug: Onboot: $vserver_new->{ONBOOT}\" >> /tmp/vps_create");
}

# Check if VPS IP-Address causes conflict:
if (($vserver_new->{ip}) || ($vserver->{ip}) || ($vserver_old->{ip})) {

    if ($vserver->{ip} ne '') { $validate_ip = $vserver->{ip}; }
    if ($vserver_old->{ip} ne '') { $validate_ip = $vserver_old->{ip}; }
    if ($vserver_new->{ip} ne '') { $validate_ip = $vserver_new->{ip}; }

        my @oids = $cce->findx("Vserver", {},
                               {
                                'ip' => $validate_ip,
                                'is_dir_present' => '1'
                               });

        # There should be no oids found
        #
        # Unfortunately both find() and findx() are stupid and do partial matches.
        # So 127.0.0.1 also matches 127.0.0.10 :o(
        # Therefore we have to do some extra magic:
        if (scalar(@oids) > 1) {
                $the_scalar = scalar(@oids);
                $out_msg = "";
                # Walk through all returned oids. One of them is the one we want to modify. Any extra result may
                # be a partial match.  
                foreach $entry (@oids) {
                        ($ok, $conflict) = $cce->get($entry);
                        # Check if a returned match has the same hostname. If so, it's the object we want to write to and which we then ignore.
                        # Compare with 'eq' to find out if it's a fully *identical* IP:
                        if (($conflict->{ip} eq $validate_ip) && ($vserver->{hostname} ne $conflict->{hostname})) {
                                $out_msg = $out_msg . $entry . " [" . $conflict->{ip} . " == " . $validate_ip . "] ";
                                # Ok, returned result is identical AND found in a VPS with differrent hostname.
                                # So we - rightfully - throw an error:
                                $cce->bye('FAIL',
                                        "[[base-vserver.ipInUse,ip='$validate_ip (create.pl:79: CCE Object ID: $out_msg - $the_scalar)']]");
                                
                                exit(1);
                        }
                }
        }

        # verify that the VPS doesn't have the same IP as the master node itself:
        my ($network) = $cce->find('Network', { 'ipaddr' => $validate_ip, 'enabled' => '1' });
        if ($network) {
                $cce->bye('FAIL',
                          "[[base-vserver.ipInUseMasterNode,ip='$validate_ip (create.pl:90: CCE Object ID: $network)']]");
                exit(1);
        }

        # Also check vpsIPalloc for IPs that may be assigned as 2ndary IPs to other VPSs:
        @oids = $cce->findx("vpsIPalloc",
                               {
                                'min' => $validate_ip
                               });

        # there should be no oids found
        if (scalar(@oids) > 0) {
                $the_scalar = scalar(@oids);
                foreach $line (@oids) {
                        $out_msg = $out_msg . " " . $line;
                }

                $cce->bye('FAIL',
                          "[[base-vserver.ipInUse,ip='$validate_ip (create.pl:108: CCE Object ID:$out_msg - $the_scalar)']]");
                exit(1);
        }
}

if (($vserver_new->{controlAction} == "create") && ($vpsid) && ($vserver_new->{ORIGIN_SAMPLE}) && ($vserver_new->{OSTEMPLATE})) {
    if ($DEBUG == 1) {
	system("echo /usr/sbin/vzctl create $vpsid --ostemplate $vserver_new->{OSTEMPLATE} --config $vserver_new->{ORIGIN_SAMPLE} >> /tmp/vps_create");
    }
    if ($vserver_new->{ONBOOT} eq "Yes") {
	$onboot_param = "yes";
    }
    else {
	$onboot_param = "no";
    }

    system("/usr/sbin/vzctl create $vpsid --ostemplate $vserver_new->{OSTEMPLATE} --config $vserver_new->{ORIGIN_SAMPLE}");

    # Set Network related stuff:
    system("/usr/sbin/vzctl set $vpsid --nameserver \"$nameservers\" --hostname $vserver_new->{hostname} --ipadd $vserver_new->{ip} --onboot $onboot_param --save");

    # Set resources:
    system("/usr/sbin/vzctl set $vpsid --physpages $vserver_new->{'physpages_b'}:$vserver_new->{'physpages_l'} --kmemsize $vserver_new->{'kmemsize_b'}:$vserver_new->{'kmemsize_l'} --dcachesize $vserver_new->{'dcachesize_b'}:$vserver_new->{'dcachesize_l'} --lockedpages $vserver_new->{'lockedpages_b'}:$vserver_new->{'lockedpages_l'} --swappages $vserver_new->{'swappages_b'}:$vserver_new->{'swappages_l'} --diskspace $vserver_new->{'diskspace_s'}:$vserver_new->{'diskspace_h'} --save");

    # If - at this point - we have something like "/vz/private/$vpsid.tmp" left around, then the VPS could not be created.
    # Possible causes are a corruption of the OS template - or more likely: VPS needed more space than granted in the sample
    # config file that was used. So exit with error code 1 and CCE msg "failOnCreate".

    if (-d "/vz/private/$vpsid.tmp") {
        $cce->bye('FAIL', "[[base-vserver.failOnCreate]]");
        exit(1);
    }
    else {
        system("/bin/chmod 755 /vz/private/$vpsid");
        system("/bin/chmod 644 /etc/vz/conf/$vpsid.conf");
        system("/usr/sbin/vzctl start $vpsid");
        #system("/usr/sausalito/handlers/vz/vserver/delayed_network_restart.pl vpsid $vpsid &");
    }
    #&cleanup;
}

sub cleanup {
    # Control action may also need to be reset to 'none' once we're done - but not on delete!
    if ($vserver_new->{controlAction}) {
	($sys_oid) = $cce->find('Vserver', {'vpsid' => $vpsid});
	($ok, $sysvs) = $cce->get($sys_oid);
	($ok) = $cce->set($sys_oid, '',{        # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
	    'vpsid' => $vpsid,
	    'controlAction' => 'none'
	});
    }
}

# We need to say 'SUCCESS' or CCE will bitch:
$cce->bye('SUCCESS');
exit(0);


