#!/usr/bin/perl -I/usr/sausalito/perl
# $Id: vps_disk_swatch.pl, v2.0.0-1 Fri 28 Oct 2011 06:07:12 AM CEST  mstauber Exp $
# Copyright 2007-2011 Stauber Multimedia Design. All rights reserved.
#
# Description:
#
# Script that runs through swatch, tests resources of a VPS and sets CCE objects 
# with status info about the VPS's. 
#

$whatami = "constructor";

use CCE;
my $cce = new CCE;
my $conf = '/var/lib/cobalt';

if ($whatami eq "handler") {
    $cce->connectfd();
}
else {
    $cce->connectuds();
}

# Setup array of return codes:
@return = ();

# Timespan in seconds between AM runs:
$span = '60';

# Location of the vzlist-dump:
$vzlistdump = '/tmp/swatch-vserver-list';

# Check if there are VPS's that we need to parse:
&check_vps_list;

# Poll CCE for all VPS's and read previous failcounts - if there are any:
if ($running_vps_present == "1") {
    &check_history;
}

# Write new status info into CCE:
if ($running_vps_present == "1") {
    &swatch_push;
}

# Return status on CLI:
$back = "GREEN";
foreach $status (@return) {
    if ($status eq "RED") { $back = "RED"; }
    if ($status eq "YELLOW") { $back = "YELLOW"; }
}
print $back . "\n";

$cce->bye('SUCCESS');
exit(0);

sub check_vps_list {
    if (( -d "/vz/private") && ( -f "/proc/vz/veinfo")) {
        system("/bin/rm -f /tmp/vps-swatch");
        system("/usr/sbin/vzlist -a --no-header | grep -v stopped | grep -v mounted | wc -l > /tmp/vps-swatch");
        open(F, "/tmp/vps-swatch");
        while ($line = <F>) {
                chomp($line);
                next if $line =~ /^\s*$/;               # skip blank lines
                if ($line =~ /^0/) {
                        $running_vps_present = "0";
                }
                else {
                        $running_vps_present = "1";
                }
        }
        close (F);
        system("/bin/rm -f /tmp/vps-swatch");
    }
}

sub swatch_push {
    # Generate vzlist-dump:
    system("vzlist -a -o vpsid,ip,hostname,status,numproc,kmemsize,kmemsize.m,kmemsize.b,kmemsize.l,kmemsize.f,lockedpages,lockedpages.m,lockedpages.b,lockedpages.l,lockedpages.f,privvmpages,privvmpages.m,privvmpages.b,privvmpages.l,privvmpages.f,shmpages,shmpages.m,shmpages.b,shmpages.l,shmpages.f,numproc,numproc.m,numproc.b,numproc.l,numproc.f,physpages,physpages.m,physpages.b,physpages.l,physpages.f,vmguarpages,vmguarpages.m,vmguarpages.b,vmguarpages.l,vmguarpages.f,oomguarpages,oomguarpages.m,oomguarpages.b,oomguarpages.l,oomguarpages.f,numtcpsock,numtcpsock.m,numtcpsock.b,numtcpsock.l,numtcpsock.f,numflock,numflock.m,numflock.b,numflock.l,numflock.f,numpty,numpty.m,numpty.b,numpty.l,numpty.f,numsiginfo,numsiginfo.m,numsiginfo.b,numsiginfo.l,numsiginfo.f,tcpsndbuf,tcpsndbuf.m,tcpsndbuf.b,tcpsndbuf.l,tcpsndbuf.f,tcprcvbuf,tcprcvbuf.m,tcprcvbuf.b,tcprcvbuf.l,tcprcvbuf.f,othersockbuf,othersockbuf.m,othersockbuf.b,othersockbuf.l,othersockbuf.f,dgramrcvbuf,dgramrcvbuf.m,dgramrcvbuf.b,dgramrcvbuf.l,dgramrcvbuf.f,numothersock,numothersock.m,numothersock.b,numothersock.l,numothersock.f,dcachesize,dcachesize.m,dcachesize.b,dcachesize.l,dcachesize.f,numfile,numfile.m,numfile.b,numfile.l,numfile.f,numiptent,numiptent.m,numiptent.b,numiptent.l,numiptent.f,diskspace,diskspace.s,diskspace.h,diskinodes,diskinodes.s,diskinodes.h,laverage,cpulimit,cpuunits | sed 's/^[ \t]*//;s/[ \t]*\$//' > $vzlistdump");

    # Pull the vzlist-dump into a Matrix:
    #
    # I LOVE matrix stuff! If there is a sledload of data to parse,
    # then there is hardly a more efficient way to do so.
    # A legend of what colums the matrix has is at the end of the code.

    open (F, $vzlistdump) || die "Could not open $vzlistdump: $!";
    $a = 0;                             # Matrix line number, whereas 0 is the first (useable) line of the vzlist dump
    while ($line = <F>) {
        chomp($line);
        next if $line =~ /^\s*$/;               # skip blank lines
        if ($line =~ /^([A-Za-z]\w*)/) {
            #$matrix_name = $1;
            $matrix_name = 'VPSID';
        }
        else {
            my (@row) = split (/\s+/, $line);
            push (@{$matrix_name}, \@row);
        }
        if ($VPSID[$a][0]) {            # Plausibility check. We will not store anything into CCE if 'vpsid' is not set.
            $currentvpsid = $VPSID[$a][0];
            &feedthemonster;            # Feed info into CCE
            if ($new_vps_found == '1') {
                &feedthemonster;                # We found a new VPS. We need to feed the monster again!
            }
        }
        $a++;
    }
    close(F);
    system("rm $vzlistdump");

}

sub feedthemonster {

	# Validate input and compare with historic data:
	&validate;

        # This is where we actually populate CCE with the gathered info:
        @oids = $cce->find('Vserver', {'vpsid' => $currentvpsid});
        if ($#oids < 0) {
                #print "Object not yet in CCE. Creating new one.\n";
                ($ok) = $cce->create('Vserver', {
                'vpsid' => $VPSID[$a][0],
                'ip' => $VPSID[$a][1],
                'hostname' => $VPSID[$a][2],
                'timestamp' => time()
                });
                $new_vps_found = '1';                   # We found a new VPS, one that we have never seen before and which
                                                        # is not yet in CCE. We do not want to risk to populate CCE with ALL
                                                        # the found new data in one go. So let us just set the most basic info
                                                        # and the timestamp.
                                                        #
                                                        # We also set $new_vps_found = '1' to run feedthemonster() again.
        }
        else {
                #print "Object already present in CCE. Updating it.\n";
                # find the system object
                $new_vps_found = '0';
                ($sys_oid) = $cce->find('Vserver', {'vpsid' => $currentvpsid});
                ($ok, $sys) = $cce->get($sys_oid);

                # For debugging and logging purpose we do not write all 122 entries into CCE in one go. Instead we break it down
                # into multiple blocks so that we can troubleshoot /var/log/messages easily for data-blocks with values that CCE
                # did not like.

                ($ok) = $cce->set($sys_oid, '',{        # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'vpsid' => $VPSID[$a][0],
                'hostname' => $VPSID[$a][2],
                'status' => $VPSID[$a][3],
                'kmemsize_f' => $VPSID[$a][9],
                'timestamp' => time()
                });

                ($ok) = $cce->set($sys_oid, '',{        # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'lockedpages_f' => $VPSID[$a][14],
                'privvmpages_f' => $VPSID[$a][19],
                'shmpages_f' => $VPSID[$a][24],
                'numproc_f' => $VPSID[$a][29],
                'physpages_f' => $VPSID[$a][34],
                'vmguarpages_f' => $VPSID[$a][39],
                'timestamp' => time()
                });

                ($ok) = $cce->set($sys_oid, '',{        # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'oomguarpages_f' => $VPSID[$a][44],
                'numtcpsock_f' => $VPSID[$a][49],
                'numflock_f' => $VPSID[$a][54],
                'numpty_f' => $VPSID[$a][59],
                'numsiginfo_f' => $VPSID[$a][64],
                'tcpsndbuf_f' => $VPSID[$a][69],
                'timestamp' => time()
                });

                ($ok) = $cce->set($sys_oid, '',{        # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'tcprcvbuf_f' => $VPSID[$a][74],
                'othersockbuf_f' => $VPSID[$a][79],
                'dgramrcvbuf_f' => $VPSID[$a][84],
                'numothersock_f' => $VPSID[$a][89],
                'dcachesize_f' => $VPSID[$a][94],
                'numfile_f' => $VPSID[$a][99],
                'timestamp' => time()
                });

                ($ok) = $cce->set($sys_oid, '',{        # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'numiptent_f' => $VPSID[$a][104],
                'diskspace' => $VPSID[$a][105],
                'diskspace_s' => $VPSID[$a][106],
                'diskspace_h' => $VPSID[$a][107],
                'timestamp' => time()
                });

                ($ok) = $cce->set($sys_oid, '',{        # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'diskinodes' => $VPSID[$a][108],
                'diskinodes_s' => $VPSID[$a][109],
                'diskinodes_h' => $VPSID[$a][110],
                'laverage' => $VPSID[$a][111],
		'failcnt' => $VPSID[$a][200],
	        'failcnt_lastevent' => $VPSID[$a][201],
	        'failcnt_general' => $VPSID[$a][202],
                'timestamp' => time()
                });
        }
}

# Check history of failcount in CCE:
sub check_history {

    @amoids = $cce->find('Vserver');
    if ($#amoids < 0) {
	# No Vserver objects in CCE. Silently exiting:
	exit 0;
    }
    else {
	foreach $entry (@amoids) {
	    ($ok, $vps_fetched) = $cce->get($entry);
            $vpsid = $vps_fetched->{'vpsid'};
            $hostname = $vps_fetched->{'hostname'};

	    $status[$vpsid] = $vps_fetched->{'status'};
	    $numproc[$vpsid] = $vps_fetched->{'numproc'};
	    $kmemsize_f[$vpsid] = $vps_fetched->{'kmemsize_f'};
	    $lockedpages_f[$vpsid] = $vps_fetched->{'lockedpages_f'};
	    $privvmpages_f[$vpsid] = $vps_fetched->{'privvmpages_f'};
	    $shmpages_f[$vpsid] = $vps_fetched->{'shmpages_f'};
	    $numproc_f[$vpsid] = $vps_fetched->{'numproc_f'};
	    $physpages_f[$vpsid] = $vps_fetched->{'physpages_f'};
	    $vmguarpages_f[$vpsid] = $vps_fetched->{'vmguarpages_f'};
	    $oomguarpages_f[$vpsid] = $vps_fetched->{'oomguarpages_f'};
	    $numtcpsock_f[$vpsid] = $vps_fetched->{'numtcpsock_f'};
	    $numflock_f[$vpsid] = $vps_fetched->{'numflock_f'};
	    $numpty_f[$vpsid] = $vps_fetched->{'numpty_f'};
	    $numsiginfo_f[$vpsid] = $vps_fetched->{'numsiginfo_f'};
	    $tcpsndbuf_f[$vpsid] = $vps_fetched->{'tcpsndbuf_f'};
	    $tcprcvbuf_f[$vpsid] = $vps_fetched->{'tcprcvbuf_f'};
	    $othersockbuf_f[$vpsid] = $vps_fetched->{'othersockbuf_f'};
	    $dgramrcvbuf_f[$vpsid] = $vps_fetched->{'dgramrcvbuf_f'};
	    $numothersock_f[$vpsid] = $vps_fetched->{'numothersock_f'};
	    $dcachesize_f[$vpsid] = $vps_fetched->{'dcachesize_f'};
	    $numfile_f[$vpsid] = $vps_fetched->{'numfile_f'};
	    $numiptent_f[$vpsid] = $vps_fetched->{'numiptent_f'};
	    $diskspace[$vpsid] = $vps_fetched->{'diskspace'};
	    $diskspace_s[$vpsid] = $vps_fetched->{'diskspace_s'};
	    $diskspace_h[$vpsid] = $vps_fetched->{'diskspace_h'};
	    $diskinodes[$vpsid] = $vps_fetched->{'diskinodes'};
	    $diskinodes_s[$vpsid] = $vps_fetched->{'diskinodes_s'};
	    $diskinodes_h[$vpsid] = $vps_fetched->{'diskinodes_h'};
	    $laverage[$vpsid] = $vps_fetched->{'laverage'};
	    $failcnt[$vpsid] = $vps_fetched->{'failcnt'};
	    $failcnt_lastevent[$vpsid] = $vps_fetched->{'failcnt_lastevent'};
	    $failcnt_general[$vpsid] = $vps_fetched->{'failcnt_general'};

	}
    }
}
sub validate {

    $my_vpsid = $VPSID[$a][0];

# Not yet included here:
#        $status[$my_vpsid]
#        $numproc[$my_vpsid]
#        $diskspace[$my_vpsid]
#        $diskspace_s[$my_vpsid]
#        $diskspace_h[$my_vpsid]
#        $diskinodes[$my_vpsid]
#        $diskinodes_s[$my_vpsid]
#        $diskinodes_h[$my_vpsid]
#        $laverage[$my_vpsid]
#	$failcnt[$my_vpsid]
#	$failcnt_lastevent[$my_vpsid]

    if ($VPSID[$a][3] == 'running') {

	$now = time();
	$firsterror = '0';
	$olderror = '0';
	$newerror = '0';
	$lastevent = $failcnt_lastevent[$my_vpsid];

	# Set lastevent to a default if it is unset.
	if (!$lastevent) { 
	    $lastevent = $now; 
	}
	
	# Detect old errors:
	if (($VPSID[$a][9] > '0') || 
    	    ($VPSID[$a][14] > '0') || 
    	    ($VPSID[$a][19] > '0') || 
    	    ($VPSID[$a][24] > '0') || 
    	    ($VPSID[$a][29] > '0') || 
    	    ($VPSID[$a][34] > '0') || 
    	    ($VPSID[$a][39] > '0') || 
    	    ($VPSID[$a][44] > '0') || 
    	    ($VPSID[$a][49] > '0') || 
    	    ($VPSID[$a][54] > '0') || 
    	    ($VPSID[$a][59] > '0') || 
    	    ($VPSID[$a][64] > '0') || 
    	    ($VPSID[$a][69] > '0') || 
    	    ($VPSID[$a][74] > '0') || 
    	    ($VPSID[$a][79] > '0') || 
    	    ($VPSID[$a][84] > '0') || 
    	    ($VPSID[$a][89] > '0') || 
    	    ($VPSID[$a][94] > '0') || 
    	    ($VPSID[$a][99] > '0') ||
    	    ($VPSID[$a][104] > '0')) 
    	    
    	    {
    		# Old errors present:
    		$olderror = '1';
    	}

	# Detect new errors since last run:
	if (($VPSID[$a][9] > $kmemsize_f[$my_vpsid]) || 
    	    ($VPSID[$a][14] > $lockedpages_f[$my_vpsid]) || 
    	    ($VPSID[$a][19] > $privvmpages_f[$my_vpsid]) || 
    	    ($VPSID[$a][24] > $shmpages_f[$my_vpsid]) || 
    	    ($VPSID[$a][29] > $numproc_f[$my_vpsid]) || 
    	    ($VPSID[$a][34] > $physpages_f[$my_vpsid]) || 
    	    ($VPSID[$a][39] > $vmguarpages_f[$my_vpsid]) || 
    	    ($VPSID[$a][44] > $oomguarpages_f[$my_vpsid]) || 
    	    ($VPSID[$a][49] > $numtcpsock_f[$my_vpsid]) || 
    	    ($VPSID[$a][54] > $numflock_f[$my_vpsid]) || 
    	    ($VPSID[$a][59] > $numpty_f[$my_vpsid]) || 
    	    ($VPSID[$a][64] > $numsiginfo_f[$my_vpsid]) || 
    	    ($VPSID[$a][69] > $tcpsndbuf_f[$my_vpsid]) || 
    	    ($VPSID[$a][74] > $tcprcvbuf_f[$my_vpsid]) || 
    	    ($VPSID[$a][79] > $othersockbuf_f[$my_vpsid]) || 
    	    ($VPSID[$a][84] > $dgramrcvbuf_f[$my_vpsid]) || 
    	    ($VPSID[$a][89] > $numothersock_f[$my_vpsid]) || 
    	    ($VPSID[$a][94] > $dcachesize_f[$my_vpsid]) || 
    	    ($VPSID[$a][99] > $numfile_f[$my_vpsid]) || 
    	    ($VPSID[$a][104] > $numiptent_f[$my_vpsid])) 
    	    
    	    {
    		# New errors present:
    		$newerror = '1';
	}

	# Push out our answers:
	if (($olderror == '0') && ($newerror == '1')) {
	    # We have a new error. Error happened for the first time.
	    #print "VPS $my_vpsid reports error for the first time. \n";
            $VPSID[$a][200] = '1';
            $VPSID[$a][201] = $now;
            $VPSID[$a][202] = '1';
            push(@return, "RED");
	}
	elsif (($olderror == '1') && ($newerror == '0')) {
	    # We HAD an error before. Now it is gone. Set to yellow:
	    #print "VPS $my_vpsid reports no failcnt increase sind last error. \n";
            $VPSID[$a][200] = '1';
            $VPSID[$a][201] = $lastevent;
            $VPSID[$a][202] = '1';
            push(@return, "YELLOW");
	}
	elsif ($olderror == '1') {	
	    # We HAD an error before. And we still have one. Set to red:
	    #print "VPS $my_vpsid reports failcnt increase since last error #2. \n";
            $VPSID[$a][200] = '1';
            $VPSID[$a][201] = $now;
            $VPSID[$a][202] = '1';
            push(@return, "RED");
	}
	else {
	    # We never had an error:
	    #print "VPS $my_vpsid never had an error and is still fine. Time is now $now\n";
            $VPSID[$a][200] = '0';
            $VPSID[$a][201] = $lastevent;
            $VPSID[$a][202] = '0';
            push(@return, "GREEN");
	}	
    }
}
